#!/usr/bin/perl

use strict;
use Test::More tests => 181-4;

# 1
BEGIN { use_ok('FDN::Sql::DB_Access'); }

#$FDN::Sql::DB_Access::debug = 1;
my $test = \&FDN::Sql::DB_Access::__parse_condition;
my @tables;
my $cond;
my $res;

# 2
@tables = ();
$cond = 'TABLE.TABLE_FIELD ~ "x%"';
$res = &$test($cond, \@tables);
is ( $res, 'TABLE.TABLE_FIELD LIKE "x%"', '__parse_condition');

# 3
is ( $#tables, 0, '__parse_condition');

# 4
is ( $tables[0], 'TABLE', '__parse_condition');

# 5
@tables = ( 'PREVIOUS' );
$cond = 'TABLE_FIELD = PREVIOUS_VAL';
$res = &$test($cond,\@tables);
is ( $res, 'TABLE.TABLE_FIELD = PREVIOUS.PREVIOUS_VAL', '__parse_condition');

# 6
is ( $#tables, 1, '__parse_condition');

# 7
is ( $tables[0], 'PREVIOUS', '__parse_condition');

# 8
is ( $tables[1], 'TABLE', '__parse_condition');

# 9
@tables = ();
$cond = 'TABLE_FIELD ~ "%%"';
$res = &$test($cond,\@tables);
is ( $res, 'none', '__parse_condition');

# 10
$test = \&FDN::Sql::DB_Access::__parse_conditions;
my @conds;
my $exp;
@tables = ();
@conds = (
	'TABLE_FIELD = PREVIOUS_VAL or TABLE_FIELD = OTHER.PREVIOUS_OTHER',
	'TABLE.TABLE_FIELD ~ "x%"',
	'TABLE.TABLE_FIELD ~ "%%"');
$exp = '((TABLE.TABLE_FIELD = PREVIOUS.PREVIOUS_VAL) OR (TABLE.TABLE_FIELD = OTHER.PREVIOUS_OTHER)) AND ((TABLE.TABLE_FIELD LIKE "x%"))';
$res = &$test(\@conds,\@tables);
is ( $res, $exp, '__parse_conditions');

# 11
is ( $#tables, 2, '__parse_conditions');

# 12
is ( $tables[0], 'TABLE', '__parse_conditions' );

# 13
is ( $tables[1], 'PREVIOUS', '__parse_conditions' );

# 14
is ( $tables[2], 'OTHER', '__parse_conditions' );

# 15
$test = \&FDN::Sql::DB_Access::__parse_insert;
my @fields;
@fields = ( 'T_A=12', 'T_B=13' );
$exp = ' ( T_A,T_B ) VALUES ( 12,13 )';
$res = &$test(\@fields,'T');
is ( $res, $exp, '__parse_insert');

# 16
@fields = ( 'A=12', 'T_B=13' );
$res = &$test(\@fields,'T');
is ( $res, $exp, '__parse_insert');

# 17
$test = \&FDN::Sql::DB_Access::__set_opt_arg;
$res = &$test('TABLE_FIELD','count',undef);
$exp = 'count(TABLE_FIELD)';
is ( $res, $exp, '__set_opt_arg');

# 18
$res = &$test('T_A,T_B,T_C,T_D','count','T_A');
$exp = 'count(T_A),T_B,T_C,T_D';
is ( $res, $exp, '__set_opt_arg');

# 19
$res = &$test($res,'max','T_D');
$exp = 'count(T_A),T_B,T_C,max(T_D)';
is ( $res, $exp, '__set_opt_arg');

# 20
$res = &$test($res,'sum','T_B');
$exp = 'count(T_A),sum(T_B),T_C,max(T_D)';
is ( $res, $exp, '__set_opt_arg');

# 21
$res = &$test($res,'min','T_C');
$exp = 'count(T_A),sum(T_B),min(T_C),max(T_D)';
is ( $res, $exp, '__set_opt_arg');

# 22
$test = \&FDN::Sql::DB_Access::__add_query_opt;
$FDN::Sql::DB_Access::first_group = 'GROUP BY';
$res = &$test('group','T_E');
$exp = ' GROUP BY T_E';
is ( $res, $exp, '__add_query_opt');

# 23
$res = &$test('limit','12');
$exp = ' LIMIT 12';
is ( $res, $exp, '__add_query_opt');

# 24
$res = &$test('limit','10,5');
$exp = ' LIMIT 10,5';
is ( $res, $exp, '__add_query_opt');

# 25
$FDN::Sql::DB_Access::first_order = 'ORDER BY';
$res = &$test('orderup','T_F');
$exp = ' ORDER BY T_F ASC';
is ( $res, $exp, '__add_query_opt');

# 26
$FDN::Sql::DB_Access::first_order = 'ORDER BY';
$res = &$test('orderdn','T_F');
$exp = ' ORDER BY T_F DESC';
is ( $res, $exp, '__add_query_opt');

# 27
$FDN::Sql::DB_Access::first_order = 'ORDER BY';
$res = &$test('orderdown','T_F');
is ( $res, $exp, '__add_query_opt');

# 28
$test = \&FDN::Sql::DB_Access::__parse_options;
my @options;
my $what;
my $table;
@options = ('count(T_A)', 'sum(T_B)', 'min(T_C)', 'max(T_D)',
	    'distinct', 'group(T_E)', 'orderup(T_A)', 'orderdn(T_B)', 'limit(2,3)');
$what = 'T_A,T_B,T_C,T_D,O.O_X';
$table = 'T,O';
$cond  = 'T.T_X=O.O_X';
$exp = 'SELECT DISTINCT count(T_A),sum(T_B),min(T_C),max(T_D),O.O_X FROM T,O WHERE T.T_X=O.O_X GROUP BY T_E ORDER BY T_A ASC , T_B DESC LIMIT 2,3';
$res = &$test(\@options,$what,$table,$cond);
is ( $res, $exp, '__parse_options');

# 29
@options = ();
$what = 'T.*';
$table = 'T';
$cond = 'none';
$res = &$test(\@options,$what,$table,$cond);
$exp = 'SELECT T.* FROM T';
is ( $res, $exp, '__parse_options');

# 30
@options = ( undef, '' );
$res = &$test(\@options,$what,$table,$cond);
is ( $res, $exp, '__parse_options');

# 31
$test = \&FDN::Sql::DB_Access::__build_sql_query;
@fields = ( 'T_A=12', 'T_B=13' );
@tables = ( 'T' );
@conds = ();
$exp = 'INSERT INTO T ( T_A,T_B ) VALUES ( 12,13 )';
$res = &$test('insert',\@fields,\@tables,\@conds,\@options);
is ( $res, $exp, '__build_sql_query/insert');

# 32
$res = &$test('insert',\@fields,\@tables,'Inutiles',\@options);
is ( $res, $exp, '__build_sql_query/insert');

# 33
$res = &$test('insert',\@fields,\@tables,'Inutiles','Bof');
is ( $res, $exp, '__build_sql_query/insert');

# 34
@conds = ( 'T.OTHER_VAL=12' );
$exp = 'UPDATE T SET T_A=12,T_B=13 WHERE ((T.OTHER_VAL = 12))';
$res = &$test('update',\@fields,\@tables,\@conds,\@options);
is ( $res, $exp, '__build_sql_query/update');

# 35
$res = &$test('update',\@fields,\@tables,\@conds,'Inutiles');
is ( $res, $exp, '__build_sql_query/update');

# 36
$exp = 'DELETE FROM T WHERE ((T.OTHER_VAL = 12))';
$res = &$test('delete',\@fields,\@tables,\@conds,\@options);
is ( $res, $exp, '__build_sql_query/delete');

# 37
$res = &$test('delete',['prout prout caca'],\@tables,\@conds,\@options);
is ( $res, $exp, '__build_sql_query/delete');

# 38
$res = &$test('delete',['prout prout caca'],\@tables,\@conds,'Ca va te faire rire...');
is ( $res, $exp, '__build_sql_query/delete');

# 39
@fields = ('T.*');
@tables = ('T');
@conds  = ('T_ID=12');
@options= ();
$res = &$test('select',\@fields,\@tables,\@conds,\@options);
$exp = "SELECT T.* FROM T WHERE ((T.T_ID = 12))";
is ( $res, $exp, '__build_sql_query/select');

# 40
@fields = ('T_NOM');
@tables = ();
@conds  = ('T_NOM~"%%"', 'T_X=U_X');
@options = ( 'orderup(T_NOM)', 'orderdown(T_X)' );
$res = &$test('select',\@fields,\@tables,\@conds,\@options);
$exp = "SELECT T_NOM FROM T,U WHERE ((T.T_X = U.U_X)) ORDER BY T_NOM ASC , T_X DESC";
is ( $res, $exp, '__build_sql_query/select');

# 41
@fields = ('T_NOM');
@tables = ();
@conds  = ('T_ID=U.T_ID','U_TYPE=\'toto\'');
@options = ();
$exp = "SELECT T_NOM FROM T,U WHERE ((T.T_ID = U.T_ID)) AND ((U.U_TYPE = 'toto'))";
$res = &$test('select',\@fields,\@tables,\@conds,\@options);
is ( $res, $exp, '__build_sql_query/select');

# 42
@conds = ('T_ID=U.T_ID','U_TYPE=\'toto\' or U_TYPE=\'titi\'');
$exp = "SELECT T_NOM FROM T,U WHERE ((T.T_ID = U.T_ID)) AND ((U.U_TYPE = 'toto') OR (U.U_TYPE = 'titi'))";
$res = &$test('select',\@fields,\@tables,\@conds,\@options);
is ( $res, $exp, '__build_sql_query/select');

# 43
$exp = "DROP TABLE T";
$res = &$test('drop','rien',['T'],'rien','rien');
is ( $res, $exp, '__build_sql_query/drop');

# 44
$test = \&FDN::Sql::DB_Access::__parse_data;
my @elems;
@elems = ('T');
@tables= ();
@fields= ();
&$test(\@elems,\@fields,\@tables);
is ( $#tables, 0, '__parse_data');

# 45
is ( $#fields, 0, '__parse_data');

# 46
is ( $tables[0], 'T', '__parse_data');

# 47
is ( $fields[0], 'T.*', '__parse_data');

# 48
@elems = ('T_NOM','U','V.U_ID');
@tables= ();
@fields= ();
&$test(\@elems,\@fields,\@tables);
is_deeply( \@tables, ['T','U','V'], '__parse_data');

# 49
is_deeply( \@fields, ['T.T_NOM', 'U.*', 'V.U_ID'], '__parse_data');

# 50
$test = \&FDN::Sql::DB_Access::__parse_update;
my @modifs;
@modifs = ( 'T.U_A=1', 'T_B=2', 'T.T_C=3' );
@tables = ();
@fields = ();
$res = &$test(\@modifs,\@fields,\@tables);
ok ( $res, '__parse_update');

# 51
is ( $#tables, 0, '__parse_update');

# 52
is ( $tables[0], 'T', '__parse_update');

# 53
is ( $#fields, 2, '__parse_update');

# 54
is_deeply ( \@fields, [ 'T.U_A=1', 'T_B=2', 'T.T_C=3' ], '__parse_update' );

# 55
@modifs = ( 'T_A=1', 'U_B=2' );
@tables = ();
@fields = ();
$res = &$test(\@modifs,\@fields,\@tables);
ok ( ! $res, '__parse_update' );

# 56
$res = db_trace(0,'append','t/trace.log');
is ( $res, 0, 'db_trace');

# 57
$res = db_trace(1,'append','t/trace.log');
is ( $res, 0, 'db_trace');

# 58
$res = db_trace(0,'append','t/trace.log');
is ( $res, 1, 'db_trace');

## 59
#$res = FDN::Sql::DB_Access::__get_script_name();
#like ( $res, qr/t\/15db-access.pl$/, '__get_script_name');
#
## 60
#FDN::Sql::DB_Access::init_dbaccess();
#like ( $FDN::Sql::DB_Access::fullname, qr/t\/15db-access.pl$/, 'init_dbaccess');
#
## 61
#ok ( defined $FDN::Sql::DB_Access::GET_DATA_REUSE->{$res}, 'init_dbaccess');
#
## 62
#is_deeply( $FDN::Sql::DB_Access::GET_DATA_REUSE->{$res}, {}, 'init_dbaccess');

SKIP: {
	if ( ! -f 't/dbc.cfg' ) {
		skip 'No mysql access',119;
	}
	# /1
	use_ok('FDN::Common::Config');

	# /2
	my $cfg = Load_Config('./t/dbc.cfg');
	is ( ref($cfg), 'HASH', 'dbc.cfg');

	# /3
	my $dbh;
	$cfg = $cfg->{'test'};
	$dbh = connect_db($cfg->{'type'}, $cfg->{'host'}, $cfg->{'name'}, $cfg->{'user'},
		$cfg->{'passwd'} );
	is ( ref($dbh), 'HASH', 'connect_db' );
	
	# /4
	ok ( disconnect_db($dbh), 'diconnect_db' );

	# /5
	$dbh = connect_db($cfg->{'type'}, $cfg->{'host'}, $cfg->{'name'}, $cfg->{'user'},
		$cfg->{'passwd'} );
	$res = sql_transaction($dbh, 'drop', 'drop table if exists T');
	is ( ref($res), 'HASH', 'sql_transaction');

	# /6
	is ( $res->{'ok'}, 1, 'sql_transaction');

	# /7
	$res = sql_transaction($dbh, 'create', 'create table T ( T_A int, T_B int )');
	is ( ref($res), 'HASH', 'sql_transaction');

	# /8
	ok ( ! $res->{'err_db'}, 'sql_transaction');

	# /9
	is ( $res->{'ok'}, 1, 'sql_transaction');

	# /10
	$res = sql_transaction($dbh,'insert','insert into T values(1,2)');
	is ( ref($res), 'HASH', 'sql_transaction');

	# /11
	ok ( $res->{'ok'}, 'sql_transaction');

	# /12
	$res = sql_transaction($dbh,'insert','insert into T values(3,4)');
	is ( ref($res), 'HASH', 'sql_transaction');

	# /13
	ok ( $res->{'ok'}, 'sql_transaction');

	# /14
	$res = sql_transaction($dbh,'select','select T.* from T order by T_A asc');
	is ( ref($res), 'HASH', 'sql_transaction');

	# /15
	ok ( $res->{'ok'}, 'sql_transaction');

	# /16
	is ( $res->{'count_rows'}, 2, 'sql_transaction');

	# /17
	is ( ref($res->{'rows'}), 'ARRAY', 'sql_transaction');

	# /18
	is ( $res->{'rows'}->[0]->{'T_A'}, 1, 'sql_transaction');
	# /19
	is ( $res->{'rows'}->[0]->{'T_B'}, 2, 'sql_transaction');
	# /20
	is ( $res->{'rows'}->[1]->{'T_A'}, 3, 'sql_transaction');
	# /21
	is ( $res->{'rows'}->[1]->{'T_B'}, 4, 'sql_transaction');

	# /22
	$res = get_data($dbh,['T_A'],[],['orderup(T_A)'],[]);
	is ( ref($res), 'HASH', 'get_data');

	# /23
	is ( $res->{'count_rows'}, 2, 'get_data');

	# /24
	is ( $res->{'rows'}->[0]->{'T_A'}, 1, 'get_data');

	# /25
	is ( $res->{'rows'}->[1]->{'T_A'}, 3, 'get_data');

	# /26
	is ( $res->{'rows'}->[0]->{'T_B'}, undef, 'get_data');

	# /27
	$res = get_data($dbh,['T'],['T_A=1'],[],[]);
	is ( ref($res), 'HASH', 'get_data');

	# /28
	ok ( $res->{'ok'}, 'get_data');

	# /29
	is ( $res->{'count_rows'}, 1, 'get_data');

	# /30
	$exp = 'SELECT T.* FROM T WHERE ((T.T_A = 1))';
	is ( $res->{'query'}, $exp, 'get_data');

	# /31
	$res = new_data($dbh,['T'],['T_A=5','T_B=6']);
	is ( ref($res), 'HASH', 'new_data');

	# /32
	ok ( $res->{'ok'}, 'new_data');

	# /33
	$exp = 'INSERT INTO T ( T_A,T_B ) VALUES ( 5,6 )';
	is ( $res->{'query'}, $exp, 'new_data');

	# /34
	$res = set_data($dbh,['T_B=7'],['T_A=5']);
	is ( ref($res), 'HASH', 'set_data');

	# /35
	ok ( $res->{'ok'}, 'set_data' );

	# /36
	$res = get_data($dbh,['T'],['T_A=5'],[],[]);
	is ( $res->{'rows'}->[0]->{'T_B'}, 7, 'set_data');

	# /37
	sql_transaction($dbh,'drop','drop table if exists U');
	sql_transaction($dbh,'create','create table U ( U_A int, U_B int)');
	$res = lock_tables($dbh,['T WRITE']);
	ok ( $res->{'ok'}, 'lock_tables');

	# /38
	$res = get_data($dbh,['U'],['U_A=12'],[],[]);
	ok ( ! $res->{'ok'}, 'lock_tables');

	# /39
	$res = lock_tables($dbh,['U WRITE']);
	ok ( $res->{'ok'}, 'lock_tables');

	# /40
	$res = get_data($dbh,['U'],['U_A=12'],[],[]);
	ok ( $res->{'ok'}, 'lock_tables');

	# /41
	$res = unlock_tables($dbh,['U WRITE']);
	ok ( $res->{'ok'}, 'lock_tables');

	# /42
	# This might work with some mysql versions!
	#$res = get_data($dbh,['U'],['U_A=12'],[],[]);
	#But this cannot, or mysql is definitively broken:
	$res = new_data($dbh,['U'],['U_A=5','U_B=6']);
	ok ( $res->{'err_db'}, 'lock_tables');
	
	# /43
	$res = unlock_tables($dbh);
	ok ( $res->{'ok'}, 'lock_tables');

	# /44
	$res = get_data($dbh,['U'],['U_A=12'],[],[]);
	ok ( $res->{'ok'}, 'lock_tables');

	# /45
	$res = lock_soft($dbh,'toto');
	ok ( $res, 'lock_soft');

	# /46
	ok ( is_locked_soft($dbh,'toto'), 'lock_soft');

	# /47
	$res = sql_transaction($dbh,'select','select IS_USED_LOCK(\'toto\') as L');
	ok ( $res->{'ok'}, 'lock_soft');

	# /48
	is ( $res->{'count_rows'}, 1, 'lock_soft' );

	# /49
	isnt ( $res->{'rows'}->[0]->{'L'}, undef, 'lock_soft' );

	# /50
	$res = lock_soft($dbh,'titi');
	ok ( ! $res, 'lock_soft' );
	
	# /51
	ok ( ! is_locked_soft($dbh,'titi'), 'lock_soft');

	# /52
	$res = sql_transaction($dbh,'select','select IS_USED_LOCK(\'toto\') as L');
	ok ( $res->{'ok'}, 'lock_soft' );

	# /53
	isnt ( $res->{'rows'}->[0]->{'L'}, undef, 'lock_soft');

	# /54
	$res = sql_transaction($dbh,'select','select IS_USED_LOCK(\'titi\') as L');
	ok ( $res->{'ok'}, 'lock_soft' );

	# /55
	is ( $res->{'rows'}->[0]->{'L'}, undef, 'lock_soft' );

	# /56
	$res = unlock_soft($dbh,'toto');
	ok ( $res, 'lock_soft');
	
	# /57
	ok ( ! is_locked_soft($dbh,'toto'), 'lock_soft');

	# /58
	ok ( ! is_locked_soft($dbh,'titi'), 'lock_soft');

	# /59
	$res = sql_transaction($dbh,'select','select IS_USED_LOCK(\'titi\') as L');
	ok ( $res->{'ok'}, 'lock_soft' );

	# /60 (-> 120)
	is ( $res->{'rows'}->[0]->{'L'}, undef, 'lock_soft' );

	# /61
	$res = unlock_soft($dbh,'titi');
	ok ( ! $res , 'lock_soft');
	
	# /62
	ok ( ! is_locked_soft($dbh,'titi'), 'lock_soft');

	# /63
	$res = is_locked_soft($dbh);
	is ( $res, '', 'lock_soft');

	# /64
	$res = get_table_create($dbh,'T');
	$exp = '(T_A int, T_B int)';
	like ( $res, qr/^\s*\(\s*`T_A`\s*int\(11\)\s*default\s*NULL\s*,\s*`T_B`\s*int\(11\)\s*default\s*NULL\s*\)/m, 'get_table_create');

	# /65
	sql_transaction($dbh,'drop','drop table if exists TEMP');
	$res = clone_table_to($dbh,'V','TEMP');
	ok ( ! $res, 'clone_table_to');

	# /66
	sql_transaction($dbh,'drop','drop table if exists V');
	$res = clone_table_to($dbh,'T','V');
	ok ( $res, 'clone_table_to');

	# /67
	$exp = get_table_create($dbh,'T');
	$res = get_table_create($dbh,'V');
	is ( $res, $exp, 'clone_table_to');

	# /68
	sql_transaction($dbh,'drop','drop table if exists OLDT');
	$res = rename_tables($dbh, [{old=>'T',new=>'OLDT'},{old=>'V',new=>'T'}]);
	ok ( $res, 'rename_tables' );

	# /69
	$res = get_table_create($dbh,'OLDT');
	is ( $res, $exp, 'rename_tables');

	# /70
	$res = get_table_create($dbh,'V');
	is ( $res, undef, 'rename_tables');
	$res = get_data($dbh,['OLDT'],[],['orderdn(T_A)'],[]);

	# /71
	ok ( $res->{'ok'}, 'rename_tables');

	# /72
	is ( $res->{'count_rows'}, 3, 'rename_tables');

	# /73
	is ( $res->{'rows'}->[0]->{'T_A'}, 5, 'rename_tables');

	# /74
	is ( $res->{'rows'}->[1]->{'T_A'}, 3, 'rename_tables');

	# /75
	is ( $res->{'rows'}->[2]->{'T_A'}, 1, 'rename_tables');

	# /76
	$res = get_data($dbh,['OLDT'],['OLDT.T_A=5'],'','');
	is ( $res->{'count_rows'}, 1, 'del_data');

	# /77
	$res = del_data($dbh,['OLDT'],['OLDT.T_A=5']);
	is ( ref($res), 'HASH', 'del_data');

	# /78
	ok ( $res->{'ok'}, 'del_data');

	# /79
	$res = get_data($dbh,['OLDT'],['OLDT.T_A=5'],'','');
	is ( $res->{'count_rows'}, 0, 'del_data');

	# /80
	$test = \&FDN::Sql::DB_Access::__incr_sequence;
	$res = &$test(0,1,0,100,0);
	is ( $res, 1, '__incr_sequence');

	# /81
	$res = &$test(12,-1,0,100,0);
	is ( $res, 11, '__incr_sequence');

	# /82
	$res = &$test(0,-1,0,100,1);
	is ( $res, 100, '__incr_sequence');

	# /83
	$res = &$test(100,1,0,100,1);
	is ( $res, 0, '__incr_sequence');

	# /84
	$res = sql_transaction($dbh,'drop','drop table if exists SEQ');
	ok ( $res->{'ok'}, 'cleanup table SEQ');

	# /85
	$res = sql_transaction($dbh,'create',
		"create table SEQ ( ".
			"SEQ_NAME varchar(20) UNIQUE NOT NULL default '', ".
			"SEQ_VALUE int(11) NOT NULL default '0', ".
			"SEQ_MINVALUE int(11) NOT NULL default '0', ".
			"SEQ_MAXVALUE int(11) NOT NULL default '0', ".
			"SEQ_INC int(11) NOT NULL default '0', ".
			"SEQ_CYCLE ENUM('-1','0','1') NOT NULL default '-1', ".
			"primary key  (SEQ_NAME) )");
	ok ( $res->{'ok'}, 'create table SEQ');

	# /86
	$res = new_data($dbh,['SEQ'],[dochar('X_ID'),0,0,999999,1,dochar(0)]);
	ok ( $res->{'ok'}, 'create sequence X_ID');

	# /87
	$res = get_data($dbh,['SEQ'],['SEQ_NAME='.dochar('X_ID')],'','');
	ok ( $res->{'ok'}, 'sequence_data');

	# /88
	is ( $res->{'count_rows'}, 1, 'sequence_data');

	# /89
	is ( $res->{'rows'}->[0]->{'SEQ_VALUE'}, 0, 'sequence_data');

	# /90
	$res = sequence_data($dbh,'X_ID');
	is ( $res, 1, 'sequence_data');

	# /91
	$res = sequence_data($dbh,'X_ID');
	is ( $res, 2, 'sequence_data');

	# /92
	$res = sequence_data($dbh,'X_ID','force',999999);
	is ( $res, 999999, 'sequence_data');

	# /93
	$res = sequence_data($dbh,'X_ID');
	is ( $res, undef, 'sequence_data');

	# /94
	$res = new_data($dbh,['SEQ'],[dochar('Y_ID'),0,0,3,1,dochar(1)]);
	ok ( $res->{'ok'}, 'create sequence Y_ID');

	# /95
	$res = sequence_data($dbh,'Y_ID');
	is ( $res, 1, 'sequence_data');

	# /96
	$res = sequence_data($dbh,'Y_ID');
	is ( $res, 2, 'sequence_data');

	# /97
	$res = sequence_data($dbh,'Y_ID');
	is ( $res, 3, 'sequence_data');

	# /98
	$res = sequence_data($dbh,'Y_ID');
	is ( $res, 0, 'sequence_data');

	# /99
	$test = \&FDN::Sql::DB_Access::__parse_conditions;
	my @values;
	@tables = ();
	@conds = (
		'TABLE_FIELD = PREVIOUS_VAL or TABLE_FIELD = OTHER.PREVIOUS_OTHER',
		'TABLE.TABLE_FIELD ~ "x%"',
		'TABLE.TABLE_FIELD ~ "%%"');
	$exp = '((TABLE.TABLE_FIELD = PREVIOUS.PREVIOUS_VAL) OR (TABLE.TABLE_FIELD = OTHER.PREVIOUS_OTHER)) AND ((TABLE.TABLE_FIELD LIKE ?))';
	$res = &$test(\@conds,\@tables,\@values);
	is ( $res, $exp, '__parse_conditions/reuse');

	# /100
	is ( $#values, 0, '__parse_conditions/reuse');

	# /101
	is_deeply( \@values, ['x%'], '__parse_conditions/reuse');

	# /102
	$res = get_data_reuse($dbh,['SEQ'],['SEQ_NAME='.dochar('X_ID')],'','');
	ok ( $res->{'ok'}, 'get_data_reuse' );

	# /103
	is ( $res->{'count_rows'}, 1, 'get_data_reuse');

	# /104
	is ( $res->{'rows'}->[0]->{'SEQ_MAXVALUE'}, 999999, 'get_data_reuse');
	
	# /105
	my $sql;
	$sql = open_sql($dbh,['SEQ'],[],['orderup(SEQ_NAME)'],undef);
	is ( ref($sql), 'HASH', 'open_sql');

	# /106
	is ( $sql->{'count_rows'}, 0, 'open_sql');

	# /107
	$res = read_sql($sql);
	is ( ref($res), 'HASH', 'read_sql');

	# /108
	is ( $res->{'SEQ_NAME'}, 'X_ID', 'read_sql');

	# /109
	is ( $res->{'SEQ_MAXVALUE'}, 999999, 'read_sql');

	# /110
	$res = read_sql($sql);
	is ( ref($res), 'HASH', 'read_sql');

	# /111
	is ( $res->{'SEQ_NAME'}, 'Y_ID', 'read_sql');

	# /112
	is ( $res->{'SEQ_MAXVALUE'}, 3, 'read_sql');

	# /113
	$res = read_sql($sql);
	is ( $res, undef, 'read_sql');

	# /114
	close_sql($sql);
	is ( $dbh->{'Verrou'}, 0, 'close_sql');

	# /115
	$sql = open_sql($dbh,['SEQ'],[],['orderup(SEQ_NAME)'],undef);
	is ( ref($sql), 'HASH', 'open_sql');

	# /116
	is ( $sql->{'count_rows'}, 0, 'open_sql');

	# /117
	$res = open_sql($dbh,['T'],[],[],[]);
	is ( $res, undef, 'open_sql');

	# /118
	$res = get_data($dbh,['T'],[],[],[]);
	is ( ref($res), 'HASH', 'open_sql');

	# /119
	ok ( ! $res->{'ok'}, 'open_sql');
	close_sql($sql);
	# Before: 62 + (/119) -> 181
	



	# Clean up
	sql_transaction($dbh,'drop','drop table if exists TEMP');
	sql_transaction($dbh,'drop','drop table if exists T');
	sql_transaction($dbh,'drop','drop table if exists U');
	sql_transaction($dbh,'drop','drop table if exists V');
	sql_transaction($dbh,'drop','drop table if exists OLDT');
	sql_transaction($dbh,'drop','drop table if exists SEQ');
	
} 



